@@ -21,6 +21,10 @@
 #include <libdvbv5/descriptors.h>
 #include <libdvbv5/dvb-fe.h>
 
+#if __GNUC__ >= 9
+#pragma GCC diagnostic ignored "-Waddress-of-packed-member"
+#endif
+
 ssize_t dvb_table_eit_init(struct dvb_v5_fe_parms *parms, const uint8_t *buf,
 		ssize_t buflen, struct dvb_table_eit **table)
 {
@@ -39,7 +43,7 @@ ssize_t dvb_table_eit_init(struct dvb_v5_fe_parms *parms, const uint8_t *buf,
 	if ((buf[0] != DVB_TABLE_EIT && buf[0] != DVB_TABLE_EIT_OTHER) &&
 		!(buf[0] >= DVB_TABLE_EIT_SCHEDULE && buf[0] <= DVB_TABLE_EIT_SCHEDULE + 0xF) &&
 		!(buf[0] >= DVB_TABLE_EIT_SCHEDULE_OTHER && buf[0] <= DVB_TABLE_EIT_SCHEDULE_OTHER + 0xF)) {
-		dvb_logerr("%s: invalid marker 0x%02x, sould be 0x%02x, 0x%02x or between 0x%02x and 0x%02x or 0x%02x and 0x%02x",
+		dvb_logerr("%s: invalid marker 0x%02x, should be 0x%02x, 0x%02x or between 0x%02x and 0x%02x or 0x%02x and 0x%02x",
 				__func__, buf[0], DVB_TABLE_EIT, DVB_TABLE_EIT_OTHER,
 				DVB_TABLE_EIT_SCHEDULE, DVB_TABLE_EIT_SCHEDULE + 0xF,
 				DVB_TABLE_EIT_SCHEDULE_OTHER, DVB_TABLE_EIT_SCHEDULE_OTHER + 0xF);
@@ -156,33 +160,32 @@ void dvb_table_eit_print(struct dvb_v5_fe_parms *parms, struct dvb_table_eit *ei
 
 void dvb_time(const uint8_t data[5], struct tm *tm)
 {
-  /* ETSI EN 300 468 V1.4.1 */
-  int year, month, day, hour, min, sec;
-  int k = 0;
-  uint16_t mjd;
+	/* ETSI EN 300 468 V1.4.1 */
+	int year, month, day, hour, min, sec;
+	int k = 0;
+	uint16_t mjd;
 
-  mjd   = *(uint16_t *) data;
-  hour  = dvb_bcd(data[2]);
-  min   = dvb_bcd(data[3]);
-  sec   = dvb_bcd(data[4]);
-  year  = ((mjd - 15078.2) / 365.25);
-  month = ((mjd - 14956.1 - (int) (year * 365.25)) / 30.6001);
-  day   = mjd - 14956 - (int) (year * 365.25) - (int) (month * 30.6001);
-  if (month == 14 || month == 15) k = 1;
-  year += k;
-  month = month - 1 - k * 12;
+	mjd   = *(uint16_t *) data;
+	hour  = dvb_bcd(data[2]);
+	min   = dvb_bcd(data[3]);
+	sec   = dvb_bcd(data[4]);
+	year  = ((mjd - 15078.2) / 365.25);
+	month = ((mjd - 14956.1 - (int) (year * 365.25)) / 30.6001);
+	day   = mjd - 14956 - (int) (year * 365.25) - (int) (month * 30.6001);
+	if (month == 14 || month == 15) k = 1;
+	year += k;
+	month = month - 1 - k * 12;
 
-  tm->tm_sec   = sec;
-  tm->tm_min   = min;
-  tm->tm_hour  = hour;
-  tm->tm_mday  = day;
-  tm->tm_mon   = month - 1;
-  tm->tm_year  = year;
-  tm->tm_isdst = 1; /* dst in effect, do not adjust */
-  mktime( tm );
+	tm->tm_sec   = sec;
+	tm->tm_min   = min;
+	tm->tm_hour  = hour;
+	tm->tm_mday  = day;
+	tm->tm_mon   = month - 1;
+	tm->tm_year  = year;
+	tm->tm_isdst = -1; /* do not adjust */
+	mktime( tm );
 }
 
-
 const char *dvb_eit_running_status_name[8] = {
 	[0] = "Undefined",
 	[1] = "Not running",
