@@ -37,7 +37,7 @@
 #include "keymap.h"
 
 #ifdef HAVE_BPF
-#include "bpf.h"
+#include <bpf/bpf.h>
 #include "bpf_load.h"
 #endif
 
@@ -55,13 +55,17 @@
 
 struct input_keymap_entry_v2 {
 #define KEYMAP_BY_INDEX	(1 << 0)
-	u_int8_t  flags;
-	u_int8_t  len;
-	u_int16_t index;
-	u_int32_t keycode;
-	u_int8_t  scancode[32];
+	uint8_t  flags;
+	uint8_t  len;
+	uint16_t index;
+	uint32_t keycode;
+	uint8_t  scancode[32];
 };
 
+#ifndef input_event_sec
+#define input_event_sec time.tv_sec
+#define input_event_usec time.tv_usec
+#endif
 
 #define IR_PROTOCOLS_USER_DIR IR_KEYTABLE_USER_DIR "/protocols"
 #define IR_PROTOCOLS_SYSTEM_DIR IR_KEYTABLE_SYSTEM_DIR "/protocols"
@@ -76,8 +80,9 @@ struct input_keymap_entry_v2 {
 #endif
 
 struct keytable_entry {
-	u_int32_t scancode;
-	u_int32_t keycode;
+	// 64 bit int which can printed with %llx
+	unsigned long long scancode;
+	uint32_t keycode;
 	struct keytable_entry *next;
 };
 
@@ -377,14 +382,16 @@ static int add_keymap(struct keymap *map, const char *fname)
 
 		protocol = parse_sysfs_protocol(map->protocol, false);
 		if (protocol == SYSFS_INVALID) {
-			struct bpf_protocol *b;
+			if (strcmp(map->protocol, "none")) {
+				struct bpf_protocol *b;
 
-			b = malloc(sizeof(*b));
-			b->name = strdup(map->protocol);
-			b->param = map->param;
-			/* steal param */
-			map->param = NULL;
-			add_bpf_protocol(b);
+				b = malloc(sizeof(*b));
+				b->name = strdup(map->protocol);
+				b->param = map->param;
+				/* steal param */
+				map->param = NULL;
+				add_bpf_protocol(b);
+			}
 		} else {
 			ch_proto |= protocol;
 		}
@@ -400,7 +407,7 @@ static int add_keymap(struct keymap *map, const char *fname)
 			if (value == -1) {
 				value = strtol(se->keycode, &p, 0);
 				if (errno || *p) {
-					fprintf(stderr, _("%s: keycode `%s' not recognised, no mapping for scancode %d\n"), fname, se->keycode, se->scancode);
+					fprintf(stderr, _("%s: keycode `%s' not recognised, no mapping for scancode 0x04%llx\n"), fname, se->keycode, (unsigned long long)se->scancode);
 					continue;
 				}
 			}
@@ -589,7 +596,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 				return ENOMEM;
 			}
 
-			ke->scancode = strtoul(p, NULL, 0);
+			ke->scancode = strtoull(p, NULL, 0);
 			if (errno) {
 				free(ke);
 				argp_error(state, _("Invalid scancode: %s"), p);
@@ -616,7 +623,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 			ke->keycode = key;
 
 			if (debug)
-				fprintf(stderr, _("scancode 0x%04x=%u\n"),
+				fprintf(stderr, _("scancode 0x%04llx=%u\n"),
 					ke->scancode, ke->keycode);
 
 			ke->next = keytable;
@@ -722,21 +729,21 @@ static struct argp argp = {
 	.doc = doc,
 };
 
-static void prtcode(int *codes)
+static void prtcode(unsigned long long scancode, int keycode)
 {
 	struct parse_event *p;
 
 	for (p = key_events; p->name != NULL; p++) {
-		if (p->value == (unsigned)codes[1]) {
-			printf(_("scancode 0x%04x = %s (0x%02x)\n"), codes[0], p->name, codes[1]);
+		if (p->value == keycode) {
+			printf(_("scancode 0x%04llx = %s (0x%02x)\n"), scancode, p->name, keycode);
 			return;
 		}
 	}
 
-	if (isprint (codes[1]))
-		printf(_("scancode 0x%04x = '%c' (0x%02x)\n"), codes[0], codes[1], codes[1]);
+	if (isprint (keycode))
+		printf(_("scancode 0x%04llx = '%c' (0x%02x)\n"), scancode, keycode, keycode);
 	else
-		printf(_("scancode 0x%04x = 0x%02x\n"), codes[0], codes[1]);
+		printf(_("scancode 0x%04llx = 0x%02x\n"), scancode, keycode);
 }
 
 static void free_names(struct sysfs_names *names)
@@ -940,12 +947,21 @@ static struct sysfs_names *find_device(char *name)
 	return names;
 }
 
+/*
+ * If an rcdev does not have a decoder for a protocol, try to load a bpf
+ * replacement.
+ */
 static enum sysfs_protocols load_bpf_for_unsupported(enum sysfs_protocols protocols, enum sysfs_protocols supported)
 {
 	const struct protocol_map_entry *pme;
 	struct bpf_protocol *b;
 
 	for (pme = protocol_map; pme->name; pme++) {
+		// So far, we only have a replacement for the xbox_dvd
+		// protocol
+		if (pme->sysfs_protocol != SYSFS_XBOX_DVD)
+			continue;
+
 		if (!(protocols & pme->sysfs_protocol) ||
 		    (supported & pme->sysfs_protocol))
 			continue;
@@ -1151,7 +1167,7 @@ static int v2_set_protocols(struct rc_device *rc_dev)
 
 	if (!stat(name, &st) && !(st.st_mode & 0222)) {
 		fprintf(stderr, _("Protocols for device can not be changed\n"));
-		return 0;
+		return EINVAL;
 	}
 
 	fp = fopen(name, "w");
@@ -1318,8 +1334,7 @@ static int set_proto(struct rc_device *rc_dev)
 	int rc = 0;
 
 	if (rc_dev->version == VERSION_2) {
-		rc = v2_set_protocols(rc_dev);
-		return rc;
+		return v2_set_protocols(rc_dev);
 	}
 
 	rc_dev->current &= rc_dev->supported;
@@ -1363,7 +1378,7 @@ static int get_input_protocol_version(int fd)
 static void clear_table(int fd)
 {
 	int i, j;
-	u_int32_t codes[2];
+	uint32_t codes[2];
 	struct input_keymap_entry_v2 entry;
 
 	/* Clears old table */
@@ -1399,17 +1414,34 @@ static int add_keys(int fd)
 	for (ke = keytable; ke; ke = ke->next) {
 		write_cnt++;
 		if (debug)
-			fprintf(stderr, "\t%04x=%04x\n",
+			fprintf(stderr, "\t%04llx=%04x\n",
 				ke->scancode, ke->keycode);
 
 		codes[0] = ke->scancode;
 		codes[1] = ke->keycode;
 
-		if (ioctl(fd, EVIOCSKEYCODE, codes)) {
-			fprintf(stderr,
-				_("Setting scancode 0x%04x with 0x%04x via "),
-				ke->scancode, ke->keycode);
-			perror("EVIOCSKEYCODE");
+		if (codes[0] != ke->scancode) {
+			// 64 bit scancode
+			struct input_keymap_entry_v2 entry = {
+				.keycode = ke->keycode,
+				.len = sizeof(ke->scancode)
+			};
+
+			memcpy(entry.scancode, &ke->scancode, sizeof(ke->scancode));
+
+			if (ioctl(fd, EVIOCSKEYCODE_V2, &entry)) {
+				fprintf(stderr,
+					_("Setting scancode 0x%04llx with 0x%04x via "),
+					ke->scancode, ke->keycode);
+				perror("EVIOCSKEYCODE");
+			}
+		} else {
+			if (ioctl(fd, EVIOCSKEYCODE, codes)) {
+				fprintf(stderr,
+					_("Setting scancode 0x%04llx with 0x%04x via "),
+					ke->scancode, ke->keycode);
+				perror("EVIOCSKEYCODE");
+			}
 		}
 	}
 
@@ -1433,7 +1465,7 @@ static void display_proto(struct rc_device *rc_dev)
 }
 
 
-static char *get_event_name(struct parse_event *event, u_int16_t code)
+static char *get_event_name(struct parse_event *event, uint16_t code)
 {
 	struct parse_event *p;
 
@@ -1533,7 +1565,7 @@ static void test_event(struct rc_device *rc_dev, int fd)
 
 		for (i = 0; i < rd / sizeof(struct input_event); i++) {
 			printf(_("%ld.%06ld: event type %s(0x%02x)"),
-				ev[i].time.tv_sec, ev[i].time.tv_usec,
+				ev[i].input_event_sec, ev[i].input_event_usec,
 				get_event_name(events_type, ev[i].type), ev[i].type);
 
 			switch (ev[i].type) {
@@ -1596,7 +1628,7 @@ static void display_table_v1(struct rc_device *rc_dev, int fd)
 			if (ioctl(fd, EVIOCGKEYCODE, codes) == -1)
 				perror("EVIOCGKEYCODE");
 			else if (codes[1] != KEY_RESERVED)
-				prtcode(codes);
+				prtcode(codes[0], codes[1]);
 		}
 	}
 	display_proto(rc_dev);
@@ -1604,25 +1636,37 @@ static void display_table_v1(struct rc_device *rc_dev, int fd)
 
 static void display_table_v2(struct rc_device *rc_dev, int fd)
 {
+	struct input_keymap_entry_v2 entry = {};
+	unsigned long long scancode;
 	int i;
-	struct input_keymap_entry_v2 entry;
-	int codes[2];
 
-	memset(&entry, '\0', sizeof(entry));
 	i = 0;
 	do {
 		entry.flags = KEYMAP_BY_INDEX;
 		entry.index = i;
-		entry.len = sizeof(u_int32_t);
+		entry.len = sizeof(scancode);
 
 		if (ioctl(fd, EVIOCGKEYCODE_V2, &entry) == -1)
 			break;
 
-		/* FIXME: Extend it to support scancodes > 32 bits */
-		memcpy(&codes[0], entry.scancode, sizeof(codes[0]));
-		codes[1] = entry.keycode;
+		if (entry.len == sizeof(uint32_t)) {
+			uint32_t temp;
 
-		prtcode(codes);
+			memcpy(&temp, entry.scancode, sizeof(temp));
+
+			scancode = temp;
+		} else if (entry.len == sizeof(uint64_t)) {
+			uint64_t temp;
+
+			memcpy(&temp, entry.scancode, sizeof(temp));
+
+			scancode = temp;
+		} else {
+			printf("error: unknown scancode length %d\n", entry.len);
+			continue;
+		}
+
+		prtcode(scancode, entry.keycode);
 		i++;
 	} while (1);
 	display_proto(rc_dev);
@@ -1954,6 +1998,38 @@ int bpf_param(struct protocol_param *protocol_param, const char *name, int *val)
 	return -ENOENT;
 }
 
+char* keymap_to_filename(const char *fname)
+{
+	struct stat st;
+	char *p;
+
+	if (fname[0] == '/' || ((fname[0] == '.') && strchr(fname, '/')))
+		return strdup(fname);
+
+	if (asprintf(&p, IR_KEYTABLE_USER_DIR "/%s", fname) < 0) {
+		fprintf(stderr, _("asprintf failed: %m\n"));
+		return NULL;
+	}
+
+	if (!stat(p, &st))
+		return p;
+
+	free(p);
+	if (asprintf(&p, IR_KEYTABLE_SYSTEM_DIR "/%s", fname) < 0) {
+		fprintf(stderr, _("asprintf failed: %m\n"));
+		return NULL;
+	}
+
+	if (!stat(p, &st))
+		return p;
+
+	free(p);
+
+	fprintf(stderr, _("error: Unable to find keymap %s in %s or %s\n"), fname, IR_KEYTABLE_USER_DIR, IR_KEYTABLE_SYSTEM_DIR);
+
+	return NULL;
+}
+
 int main(int argc, char *argv[])
 {
 	int dev_from_class = 0, write_cnt;
@@ -2003,6 +2079,7 @@ int main(int argc, char *argv[])
 
 	if (cfg.next) {
 		struct cfgfile *cur;
+		struct keymap *map;
 		char *fname;
 		int rc;
 		int matches = 0;
@@ -2014,53 +2091,30 @@ int main(int argc, char *argv[])
 				continue;
 
 			if (debug)
-				fprintf(stderr, _("Table for %s, %s is on %s file.\n"),
+				fprintf(stderr, _("Keymap for %s, %s is on %s file.\n"),
 					rc_dev.drv_name, rc_dev.keytable_name,
 					cur->fname);
 
-			if (cur->fname[0] == '/' || ((cur->fname[0] == '.') && strchr(cur->fname, '/'))) {
-				struct keymap *map;
-				fname = cur->fname;
-				rc = parse_keymap(fname, &map, debug);
-				if (rc < 0) {
-					fprintf(stderr, _("Can't load %s table\n"), fname);
-					return -1;
-				}
-				add_keymap(map, fname);
-				free_keymap(map);
-			} else {
-				struct keymap *map;
-				fname = malloc(strlen(cur->fname) + strlen(IR_KEYTABLE_USER_DIR) + 2);
-				strcpy(fname, IR_KEYTABLE_USER_DIR);
-				strcat(fname, "/");
-				strcat(fname, cur->fname);
-				rc = parse_keymap(fname, &map, debug);
-				if (rc != 0) {
-					fname = malloc(strlen(cur->fname) + strlen(IR_KEYTABLE_SYSTEM_DIR) + 2);
-					strcpy(fname, IR_KEYTABLE_SYSTEM_DIR);
-					strcat(fname, "/");
-					strcat(fname, cur->fname);
-					rc = parse_keymap(fname, &map, debug);
-				}
-				if (rc != 0) {
-					fprintf(stderr, _("Can't load %s table from %s or %s\n"), cur->fname, IR_KEYTABLE_USER_DIR, IR_KEYTABLE_SYSTEM_DIR);
-					return -1;
-				}
+			fname = keymap_to_filename(cur->fname);
+			if (!fname)
+				return -1;
 
-				add_keymap(map, fname);
-				free_keymap(map);
-			}
-			if (!keytable) {
-				fprintf(stderr, _("Empty table %s\n"), fname);
+			rc = parse_keymap(fname, &map, debug);
+			if (rc < 0) {
+				fprintf(stderr, _("Can't load %s keymap\n"), fname);
+				free(fname);
 				return -1;
 			}
+			add_keymap(map, fname);
+			free_keymap(map);
+			free(fname);
 			clear = 1;
 			matches++;
 		}
 
 		if (!matches) {
 			if (debug)
-				fprintf(stderr, _("Table for %s, %s not found. Keep as-is\n"),
+				fprintf(stderr, _("Keymap for %s, %s not found. Keep as-is\n"),
 				       rc_dev.drv_name, rc_dev.keytable_name);
 			return 0;
 		}
@@ -2102,9 +2156,7 @@ int main(int argc, char *argv[])
 
 		rc_dev.current = load_bpf_for_unsupported(ch_proto, rc_dev.supported);
 
-		if (set_proto(&rc_dev))
-			fprintf(stderr, _("Couldn't change the IR protocols\n"));
-		else {
+		if (!set_proto(&rc_dev)) {
 			fprintf(stderr, _("Protocols changed to "));
 			write_sysfs_protocols(rc_dev.current, stderr, "%s ");
 			fprintf(stderr, "\n");
