@@ -28,6 +28,8 @@
 #include <QVBoxLayout>
 #include <QApplication>
 #include <QDesktopWidget>
+#include <QScreen>
+#include <QWindow>
 
 #include <math.h>
 
@@ -42,9 +44,6 @@ CaptureWin::CaptureWin(ApplicationWindow *aw) :
 	m_appWin(aw)
 {
 	setWindowTitle("V4L2 Capture");
-	m_hotkeyClose = new QShortcut(Qt::CTRL+Qt::Key_W, this);
-	connect(m_hotkeyClose, SIGNAL(activated()), this, SLOT(close()));
-	connect(new QShortcut(Qt::Key_Q, this), SIGNAL(activated()), this, SLOT(close()));
 	m_hotkeyScaleReset = new QShortcut(Qt::CTRL+Qt::Key_F, this);
 	connect(m_hotkeyScaleReset, SIGNAL(activated()), this, SLOT(resetSize()));
 	connect(aw->m_resetScalingAct, SIGNAL(triggered()), this, SLOT(resetSize()));
@@ -53,9 +52,25 @@ CaptureWin::CaptureWin(ApplicationWindow *aw) :
 	m_hotkeyToggleFullscreen = new QShortcut(Qt::Key_F, this);
 	connect(m_hotkeyToggleFullscreen, SIGNAL(activated()), aw->m_makeFullScreenAct, SLOT(toggle()));
 	m_exitFullScreen = new QAction(QIcon(":/fullscreenexit.png"), "Exit Fullscreen", this);
+	m_exitFullScreen->setShortcut(m_hotkeyToggleFullscreen->key());
 	connect(m_exitFullScreen, SIGNAL(triggered()), this, SLOT(escape()));
 	m_enterFullScreen = new QAction(QIcon(":/fullscreen.png"), "Show Fullscreen", this);
+	m_enterFullScreen ->setShortcut(m_hotkeyToggleFullscreen->key());
 	connect(m_enterFullScreen, SIGNAL(triggered()), this, SLOT(fullScreen()));
+	// Add the action to allow the hotkey to start/stop the stream
+	addAction(m_appWin->m_capStartAct);
+
+	m_closeWindowAct = new QAction(QIcon(":/fileclose.png"), "&Close Window", this);
+	m_closeWindowAct->setStatusTip("Close");
+	QList<QKeySequence> shortcuts;
+	// More standard close window shortcut
+	shortcuts << Qt::CTRL+Qt::Key_W;
+	// Historic qv4l2 shortcut
+	shortcuts << Qt::Key_Q;
+	m_closeWindowAct->setShortcuts(shortcuts);
+	addAction(m_closeWindowAct);
+	connect(m_closeWindowAct, SIGNAL(triggered()), this, SLOT(close()));
+
 	m_frame.format = 0;
 	m_frame.size.setWidth(0);
 	m_frame.size.setHeight(0);
@@ -82,7 +97,6 @@ CaptureWin::~CaptureWin()
 
 	layout()->removeWidget(this);
 	delete layout();
-	delete m_hotkeyClose;
 	delete m_hotkeyScaleReset;
 }
 
@@ -238,8 +252,19 @@ void CaptureWin::setWindowSize(QSize frameSize)
 	m_origFrameSize = frameSize;
 
 	QSize margins = getMargins();
+#if QT_VERSION < 0x050a00
 	QDesktopWidget *screen = QApplication::desktop();
 	QRect resolution = screen->screenGeometry();
+#else
+	QSize resolution(1920, 1080);
+
+	// window()->windowHandle() can be NULL, for example when
+	// using X11 forwarding.
+	if (window()->windowHandle()) {
+		QScreen *screen = window()->windowHandle()->screen();
+		resolution = screen->availableSize();
+	}
+#endif
 
 	QSize windowSize =  pixelAspectFrameSize(cropSize(frameSize)) + margins;
 
@@ -352,15 +377,20 @@ void CaptureWin::customMenuRequested(QPoint pos)
 		menu->addAction(m_enterFullScreen);
 	}
 	
+	menu->addAction(m_appWin->m_capStartAct);
+	menu->addAction(m_appWin->m_capStepAct);
 	menu->addAction(m_appWin->m_resetScalingAct);
-	menu->addAction(m_appWin->m_useBlendingAct);
-	menu->addAction(m_appWin->m_useLinearAct);
+	if (m_appWin->m_useBlendingAct)
+		menu->addAction(m_appWin->m_useBlendingAct);
+	if (m_appWin->m_useLinearAct)
+		menu->addAction(m_appWin->m_useLinearAct);
 	menu->addAction(m_appWin->m_snapshotAct);
 	menu->addAction(m_appWin->m_showFramesAct);
 	menu->addMenu(m_appWin->m_overrideColorspaceMenu);
 	menu->addMenu(m_appWin->m_overrideXferFuncMenu);
 	menu->addMenu(m_appWin->m_overrideYCbCrEncMenu);
 	menu->addMenu(m_appWin->m_overrideQuantizationMenu);
+	menu->addAction(m_closeWindowAct);
 	
 	menu->popup(mapToGlobal(pos));
 }
