@@ -57,6 +57,8 @@ const __u32 formats[] = {
 	V4L2_PIX_FMT_XYUV32,
 	V4L2_PIX_FMT_VUYA32,
 	V4L2_PIX_FMT_VUYX32,
+	V4L2_PIX_FMT_YUVA32,
+	V4L2_PIX_FMT_YUVX32,
 	V4L2_PIX_FMT_RGB32,
 	V4L2_PIX_FMT_XRGB32,
 	V4L2_PIX_FMT_ARGB32,
@@ -543,7 +545,7 @@ void CaptureWin::mouseDoubleClickEvent(QMouseEvent * e)
 	toggleFullScreen();
 }
 
-void CaptureWin::cycleMenu(__u32 &overrideVal, __u32 &origVal,
+void CaptureWin::cycleMenu(__u32 &overrideVal, __u32 origVal,
 			     const __u32 values[], bool hasShift, bool hasCtrl)
 {
 	unsigned i;
@@ -882,6 +884,8 @@ bool CaptureWin::updateV4LFormat(const cv4l_fmt &fmt)
 	case V4L2_PIX_FMT_XYUV32:
 	case V4L2_PIX_FMT_VUYA32:
 	case V4L2_PIX_FMT_VUYX32:
+	case V4L2_PIX_FMT_YUVA32:
+	case V4L2_PIX_FMT_YUVX32:
 		m_is_rgb = false;
 		m_accepts_srgb = false;
 		break;
@@ -1106,7 +1110,7 @@ bool CaptureWin::setV4LFormat(cv4l_fmt &fmt)
 	return true;
 }
 
-void CaptureWin::setPixelAspect(v4l2_fract &pixelaspect)
+void CaptureWin::setPixelAspect(const v4l2_fract &pixelaspect)
 {
 	m_pixelaspect = pixelaspect;
 }
