@@ -48,7 +48,7 @@ void CaptureWin::initializeGL()
 	}
 	if (m_uses_gl_red && glGetString(GL_VERSION)[0] < '3') {
 		fprintf(stderr, "The openGL implementation does not support GL_RED/GL_RG\n");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	QColor bg = QWidget::palette().color(QWidget::backgroundRole());
@@ -159,6 +159,8 @@ void CaptureWin::paintGL()
 	case V4L2_PIX_FMT_XYUV32:
 	case V4L2_PIX_FMT_VUYA32:
 	case V4L2_PIX_FMT_VUYX32:
+	case V4L2_PIX_FMT_YUVA32:
+	case V4L2_PIX_FMT_YUVX32:
 		render_YUV_packed(m_v4l_fmt.g_pixelformat());
 		break;
 
@@ -355,6 +357,8 @@ static const struct define defines[] = {
 	DEF(V4L2_PIX_FMT_XYUV32),
 	DEF(V4L2_PIX_FMT_VUYA32),
 	DEF(V4L2_PIX_FMT_VUYX32),
+	DEF(V4L2_PIX_FMT_YUVA32),
+	DEF(V4L2_PIX_FMT_YUVX32),
 	DEF(V4L2_PIX_FMT_RGB32),
 	DEF(V4L2_PIX_FMT_XRGB32),
 	DEF(V4L2_PIX_FMT_ARGB32),
@@ -515,7 +519,7 @@ void CaptureWin::changeShader()
 
 	if (!src_ok) {
 		fprintf(stderr, "OpenGL Error: fragment shader compilation failed.\n");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	// Mandatory vertex shader replaces fixed pipeline in GLES 2.0. In this case just a feedthrough shader.
@@ -541,12 +545,12 @@ void CaptureWin::changeShader()
 
 	if (!src_ok) {
 		fprintf(stderr, "OpenGL Error: vertex shader compilation failed.\n");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	if (!m_program->bind()) {
 		fprintf(stderr, "OpenGL Error: shader bind failed.\n");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	GLint loc = m_program->uniformLocation("uvtex");
@@ -595,6 +599,8 @@ void CaptureWin::changeShader()
 	case V4L2_PIX_FMT_XYUV32:
 	case V4L2_PIX_FMT_VUYA32:
 	case V4L2_PIX_FMT_VUYX32:
+	case V4L2_PIX_FMT_YUVA32:
+	case V4L2_PIX_FMT_YUVX32:
 		shader_YUV_packed();
 		break;
 
@@ -945,6 +951,8 @@ void CaptureWin::shader_YUV_packed()
 	case V4L2_PIX_FMT_XYUV32:
 	case V4L2_PIX_FMT_VUYA32:
 	case V4L2_PIX_FMT_VUYX32:
+	case V4L2_PIX_FMT_YUVA32:
+	case V4L2_PIX_FMT_YUVX32:
 		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, m_v4l_fmt.g_width(), m_v4l_fmt.g_height(), 0,
 			     GL_RGBA, GL_UNSIGNED_BYTE, NULL);
 		break;
@@ -1260,6 +1268,7 @@ void CaptureWin::render_Bayer(__u32 format)
 	case V4L2_PIX_FMT_SGBRG8:
 	case V4L2_PIX_FMT_SGRBG8:
 	case V4L2_PIX_FMT_SRGGB8:
+		glPixelStorei(GL_UNPACK_ROW_LENGTH, m_v4l_fmt.g_bytesperline());
 		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, m_v4l_fmt.g_width(), m_v4l_fmt.g_height(),
 				GL_RED_INTEGER, GL_UNSIGNED_BYTE, m_curData[0]);
 		break;
@@ -1275,6 +1284,7 @@ void CaptureWin::render_Bayer(__u32 format)
 	case V4L2_PIX_FMT_SGBRG16:
 	case V4L2_PIX_FMT_SGRBG16:
 	case V4L2_PIX_FMT_SRGGB16:
+		glPixelStorei(GL_UNPACK_ROW_LENGTH, m_v4l_fmt.g_bytesperline() / 2);
 		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, m_v4l_fmt.g_width(), m_v4l_fmt.g_height(),
 				GL_RED_INTEGER, GL_UNSIGNED_SHORT, m_curData[0]);
 		break;
@@ -1308,6 +1318,8 @@ void CaptureWin::render_YUV_packed(__u32 format)
 	case V4L2_PIX_FMT_XYUV32:
 	case V4L2_PIX_FMT_VUYA32:
 	case V4L2_PIX_FMT_VUYX32:
+	case V4L2_PIX_FMT_YUVA32:
+	case V4L2_PIX_FMT_YUVX32:
 		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, m_v4l_fmt.g_width(), m_v4l_fmt.g_height(),
 				GL_RGBA, GL_UNSIGNED_BYTE, m_curData[0]);
 		break;
