@@ -18,33 +18,21 @@
     Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
  */
 
-#include <cstdlib>
+#include <cctype>
+#include <csignal>
 #include <cstring>
+#include <map>
+#include <set>
+#include <vector>
 
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <inttypes.h>
+#include <dirent.h>
 #include <getopt.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <errno.h>
-#include <sys/ioctl.h>
-#include <sys/time.h>
-#include <sys/sysmacros.h>
-#include <dirent.h>
-#include <math.h>
-#include <sys/utsname.h>
-#include <signal.h>
-#include <vector>
+#include <unistd.h>
 
 #include "v4l2-compliance.h"
 #include <media-info.h>
-#ifndef ANDROID
-#include "version.h"
-#endif
+#include <v4l-getsubopt.h>
 
 /* Short option list
 
@@ -79,6 +67,7 @@ enum Option {
 	OptMediaBusInfo = 'z',
 	OptStreamFrom = 128,
 	OptStreamFromHdr,
+	OptVersion,
 	OptLast = 256
 };
 
@@ -96,9 +85,10 @@ bool show_colors;
 bool exit_on_fail;
 bool exit_on_warn;
 bool is_vivid;
-int kernel_version;
+bool is_uvcvideo;
 int media_fd = -1;
 unsigned warnings;
+bool has_mmu = true;
 
 static unsigned color_component;
 static unsigned color_skip;
@@ -127,36 +117,62 @@ struct dev_state {
 static struct dev_state state;
 
 static struct option long_options[] = {
-	{"device", required_argument, 0, OptSetDevice},
-	{"radio-device", required_argument, 0, OptSetRadioDevice},
-	{"vbi-device", required_argument, 0, OptSetVbiDevice},
-	{"sdr-device", required_argument, 0, OptSetSWRadioDevice},
-	{"subdev-device", required_argument, 0, OptSetSubDevDevice},
-	{"expbuf-device", required_argument, 0, OptSetExpBufDevice},
-	{"touch-device", required_argument, 0, OptSetTouchDevice},
-	{"media-device", required_argument, 0, OptSetMediaDevice},
-	{"media-device-only", required_argument, 0, OptSetMediaDeviceOnly},
-	{"media-bus-info", required_argument, 0, OptMediaBusInfo},
-	{"help", no_argument, 0, OptHelp},
-	{"verbose", no_argument, 0, OptVerbose},
-	{"color", required_argument, 0, OptColor},
-	{"no-warnings", no_argument, 0, OptNoWarnings},
-	{"no-progress", no_argument, 0, OptNoProgress},
-	{"exit-on-fail", no_argument, 0, OptExitOnFail},
-	{"exit-on-warn", no_argument, 0, OptExitOnWarn},
-	{"trace", no_argument, 0, OptTrace},
+	{"device", required_argument, nullptr, OptSetDevice},
+	{"radio-device", required_argument, nullptr, OptSetRadioDevice},
+	{"vbi-device", required_argument, nullptr, OptSetVbiDevice},
+	{"sdr-device", required_argument, nullptr, OptSetSWRadioDevice},
+	{"subdev-device", required_argument, nullptr, OptSetSubDevDevice},
+	{"expbuf-device", required_argument, nullptr, OptSetExpBufDevice},
+	{"touch-device", required_argument, nullptr, OptSetTouchDevice},
+	{"media-device", required_argument, nullptr, OptSetMediaDevice},
+	{"media-device-only", required_argument, nullptr, OptSetMediaDeviceOnly},
+	{"media-bus-info", required_argument, nullptr, OptMediaBusInfo},
+	{"help", no_argument, nullptr, OptHelp},
+	{"verbose", no_argument, nullptr, OptVerbose},
+	{"color", required_argument, nullptr, OptColor},
+	{"no-warnings", no_argument, nullptr, OptNoWarnings},
+	{"no-progress", no_argument, nullptr, OptNoProgress},
+	{"exit-on-fail", no_argument, nullptr, OptExitOnFail},
+	{"exit-on-warn", no_argument, nullptr, OptExitOnWarn},
+	{"trace", no_argument, nullptr, OptTrace},
 #ifndef NO_LIBV4L2
-	{"wrapper", no_argument, 0, OptUseWrapper},
+	{"wrapper", no_argument, nullptr, OptUseWrapper},
 #endif
-	{"streaming", optional_argument, 0, OptStreaming},
-	{"stream-from", required_argument, 0, OptStreamFrom},
-	{"stream-from-hdr", required_argument, 0, OptStreamFromHdr},
-	{"stream-all-formats", optional_argument, 0, OptStreamAllFormats},
-	{"stream-all-io", no_argument, 0, OptStreamAllIO},
-	{"stream-all-color", required_argument, 0, OptStreamAllColorTest},
-	{0, 0, 0, 0}
+	{"streaming", optional_argument, nullptr, OptStreaming},
+	{"stream-from", required_argument, nullptr, OptStreamFrom},
+	{"stream-from-hdr", required_argument, nullptr, OptStreamFromHdr},
+	{"stream-all-formats", optional_argument, nullptr, OptStreamAllFormats},
+	{"stream-all-io", no_argument, nullptr, OptStreamAllIO},
+	{"stream-all-color", required_argument, nullptr, OptStreamAllColorTest},
+	{"version", no_argument, nullptr, OptVersion},
+	{nullptr, 0, nullptr, 0}
 };
 
+#define STR(x) #x
+#define STRING(x) STR(x)
+
+static void print_sha()
+{
+	int fd = open("/dev/null", O_RDONLY);
+
+	if (fd >= 0) {
+		// FIONBIO is a write-only ioctl that takes an int argument that
+		// enables (!= 0) or disables (== 0) nonblocking mode of a fd.
+		//
+		// Passing a nullptr should return EFAULT on MMU capable machines,
+		// and it works if there is no MMU.
+		has_mmu = ioctl(fd, FIONBIO, nullptr);
+		close(fd);
+	}
+	printf("v4l2-compliance %s%s, ", PACKAGE_VERSION, STRING(GIT_COMMIT_CNT));
+	printf("%zd bits, %zd-bit time_t%s\n",
+	       sizeof(void *) * 8, sizeof(time_t) * 8,
+	       has_mmu ? "" : ", has no MMU");
+	if (strlen(STRING(GIT_SHA)))
+		printf("v4l2-compliance SHA: %s %s\n",
+		       STRING(GIT_SHA), STRING(GIT_COMMIT_DATE));
+}
+
 static void usage()
 {
 	printf("Usage:\n");
@@ -245,6 +261,7 @@ static void usage()
 	printf("  -P, --no-progress  Turn off progress messages.\n");
 	printf("  -T, --trace        Trace all called ioctls.\n");
 	printf("  -v, --verbose      Turn on verbose reporting.\n");
+	printf("  --version          Show version information.\n");
 #ifndef NO_LIBV4L2
 	printf("  -w, --wrapper      Use the libv4l2 wrapper library.\n");
 #endif
@@ -531,9 +548,11 @@ static void determine_codec_mask(struct node &node)
 			case V4L2_PIX_FMT_FWHT:
 				mask |= STATEFUL_ENCODER;
 				break;
+#if 0 	// There are no stateless encoders (yet)
 			case V4L2_PIX_FMT_MPEG2_SLICE:
 				mask |= STATELESS_ENCODER;
 				break;
+#endif
 			default:
 				return;
 			}
@@ -570,6 +589,10 @@ static void determine_codec_mask(struct node &node)
 				mask |= STATEFUL_DECODER;
 				break;
 			case V4L2_PIX_FMT_MPEG2_SLICE:
+			case V4L2_PIX_FMT_H264_SLICE:
+			case V4L2_PIX_FMT_HEVC_SLICE:
+			case V4L2_PIX_FMT_VP8_FRAME:
+			case V4L2_PIX_FMT_VP9_FRAME:
 			case V4L2_PIX_FMT_FWHT_STATELESS:
 				mask |= STATELESS_DECODER;
 				break;
@@ -615,9 +638,9 @@ static int testCap(struct node *node)
 		V4L2_CAP_VIDEO_M2M;
 
 	memset(&vcap, 0xff, sizeof(vcap));
-	// Must always be there
-	fail_on_test(doioctl(node, VIDIOC_QUERYCAP, NULL) != EFAULT);
 	fail_on_test(doioctl(node, VIDIOC_QUERYCAP, &vcap));
+	if (has_mmu)
+		fail_on_test(doioctl(node, VIDIOC_QUERYCAP, nullptr) != EFAULT);
 	fail_on_test(check_ustring(vcap.driver, sizeof(vcap.driver)));
 	fail_on_test(check_ustring(vcap.card, sizeof(vcap.card)));
 	fail_on_test(check_ustring(vcap.bus_info, sizeof(vcap.bus_info)));
@@ -629,19 +652,21 @@ static int testCap(struct node *node)
 	    memcmp(vcap.bus_info, "I2C:", 4) &&
 	    memcmp(vcap.bus_info, "parport", 7) &&
 	    memcmp(vcap.bus_info, "platform:", 9) &&
-	    memcmp(vcap.bus_info, "rmi4:", 5))
+	    memcmp(vcap.bus_info, "rmi4:", 5) &&
+	    memcmp(vcap.bus_info, "libcamera:", 10))
 		return fail("missing bus_info prefix ('%s')\n", vcap.bus_info);
 	if (!node->media_bus_info.empty() &&
 	    node->media_bus_info != std::string(reinterpret_cast<const char *>(vcap.bus_info)))
 		warn("media bus_info '%s' differs from V4L2 bus_info '%s'\n",
 		     node->media_bus_info.c_str(), vcap.bus_info);
 	fail_on_test((vcap.version >> 16) < 3);
+	if (vcap.version >= 0x050900)  // Present from 5.9.0 onwards
+		node->might_support_cache_hints = true;
 	fail_on_test(check_0(vcap.reserved, sizeof(vcap.reserved)));
 	caps = vcap.capabilities;
 	dcaps = vcap.device_caps;
 	node->is_m2m = dcaps & m2m_caps;
 	fail_on_test(caps == 0);
-	fail_on_test(caps & V4L2_CAP_ASYNCIO);
 	fail_on_test(!(caps & V4L2_CAP_DEVICE_CAPS));
 	fail_on_test(dcaps & V4L2_CAP_DEVICE_CAPS);
 	fail_on_test(dcaps & ~caps);
@@ -772,9 +797,9 @@ static int testInvalidIoctls(struct node *node, char type)
 	unsigned ioc = _IOC(_IOC_NONE, type, 0xff, 0);
 	unsigned char buf[0x4000] = {};
 
-	fail_on_test(doioctl(node, ioc, NULL) != ENOTTY);
+	fail_on_test(doioctl(node, ioc, nullptr) != ENOTTY);
 	ioc = _IOC(_IOC_NONE, type, 0, 0x3fff);
-	fail_on_test(doioctl(node, ioc, NULL) != ENOTTY);
+	fail_on_test(doioctl(node, ioc, nullptr) != ENOTTY);
 	ioc = _IOC(_IOC_READ, type, 0, 0x3fff);
 	fail_on_test(doioctl(node, ioc, buf) != ENOTTY);
 	fail_on_test(check_0(buf, sizeof(buf)));
@@ -810,13 +835,13 @@ static void streamingSetup(struct node *node)
 
 static int parse_subopt(char **subs, const char * const *subopts, char **value)
 {
-	int opt = getsubopt(subs, const_cast<char * const *>(subopts), value);
+	int opt = v4l_getsubopt(subs, const_cast<char * const *>(subopts), value);
 
 	if (opt == -1) {
 		fprintf(stderr, "Invalid suboptions specified\n");
 		return -1;
 	}
-	if (*value == NULL) {
+	if (*value == nullptr) {
 		fprintf(stderr, "No value given to suboption <%s>\n",
 				subopts[opt]);
 		return -1;
@@ -830,7 +855,7 @@ static int open_media_bus_info(const std::string &bus_info, std::string &media_d
 	struct dirent *ep;
 
 	dp = opendir("/dev");
-	if (dp == NULL)
+	if (dp == nullptr)
 		return -1;
 
 	while ((ep = readdir(dp))) {
@@ -883,11 +908,11 @@ static std::string make_devname(const char *device, const char *devname,
 		return device;
 	}
 
-	media_v2_entity *ents = new media_v2_entity[topology.num_entities];
+	auto ents = new media_v2_entity[topology.num_entities];
 	topology.ptr_entities = (uintptr_t)ents;
-	media_v2_link *links = new media_v2_link[topology.num_links];
+	auto links = new media_v2_link[topology.num_links];
 	topology.ptr_links = (uintptr_t)links;
-	media_v2_interface *ifaces = new media_v2_interface[topology.num_interfaces];
+	auto ifaces = new media_v2_interface[topology.num_interfaces];
 	topology.ptr_interfaces = (uintptr_t)ifaces;
 
 	unsigned i, ent_id, iface_id = 0;
@@ -898,7 +923,7 @@ static std::string make_devname(const char *device, const char *devname,
 		goto err;
 
 	if (device[0] == '0' && device[1] == 'x')
-		iface_id = strtoul(device, NULL, 16);
+		iface_id = strtoul(device, nullptr, 16);
 
 	if (!iface_id) {
 		for (i = 0; i < topology.num_entities; i++)
@@ -936,7 +961,8 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 	      unsigned frame_count, unsigned all_fmt_frame_count)
 {
 	struct node node2;
-	struct v4l2_capability vcap;		/* list_cap */
+	struct v4l2_capability vcap = {};
+	struct v4l2_subdev_capability subdevcap = {};
 	std::string driver;
 
 	tests_total = tests_ok = warnings = 0;
@@ -950,10 +976,13 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 	if (node.is_v4l2()) {
 		doioctl(&node, VIDIOC_QUERYCAP, &vcap);
 		driver = reinterpret_cast<const char *>(vcap.driver);
+		is_uvcvideo = driver == "uvcvideo";
 		is_vivid = driver == "vivid";
 		if (is_vivid)
 			node.bus_info = reinterpret_cast<const char *>(vcap.bus_info);
 		determine_codec_mask(node);
+	} else if (node.is_subdev()) {
+		doioctl(&node, VIDIOC_SUBDEV_QUERYCAP, &subdevcap);
 	} else {
 		memset(&vcap, 0, sizeof(vcap));
 	}
@@ -969,15 +998,15 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 			if (!node.is_v4l2())
 				driver = mdinfo.driver;
 			node.media_bus_info = mdinfo.bus_info;
+			node.has_media = true;
 		}
 	}
 
 	if (driver.empty())
-		printf("Compliance test for device %s%s:\n\n",
-		       node.device, node.g_direct() ? "" : " (using libv4l2)");
+		printf("Compliance test for device ");
 	else
-		printf("Compliance test for %s device %s%s:\n\n",
-		       driver.c_str(), node.device, node.g_direct() ? "" : " (using libv4l2)");
+		printf("Compliance test for %s device ", driver.c_str());
+	printf("%s%s:\n\n", node.device, node.g_direct() ? "" : " (using libv4l2)");
 
 	if (node.g_caps() & (V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_VBI_CAPTURE |
 			 V4L2_CAP_VIDEO_CAPTURE_MPLANE | V4L2_CAP_SLICED_VBI_CAPTURE |
@@ -1034,6 +1063,9 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 			printf("\tDetected Stateless Decoder\n");
 			break;
 		}
+	} else if (node.is_subdev()) {
+		printf("Driver Info:\n");
+		v4l2_info_subdev_capability(subdevcap);
 	}
 
 	__u32 ent_id = 0;
@@ -1042,7 +1074,7 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 	if (node.is_media())
 		mi_media_info_for_fd(node.g_fd(), -1, &is_invalid);
 	else if (media_fd >= 0)
-		ent_id = mi_media_info_for_fd(media_fd, node.g_fd(), &is_invalid);
+		ent_id = mi_media_info_for_fd(media_fd, node.g_fd(), &is_invalid, &node.function);
 
 	if (ent_id != MEDIA_ENT_F_UNKNOWN) {
 		memset(&node.entity, 0, sizeof(node.entity));
@@ -1072,10 +1104,16 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 
 	if (node.is_v4l2())
 		printf("\ttest VIDIOC_QUERYCAP: %s\n", ok(testCap(&node)));
+	else if (node.is_subdev())
+		printf("\ttest VIDIOC_SUDBEV_QUERYCAP: %s\n", ok(testSubDevCap(&node)));
+
+	if (node.is_v4l2() || node.is_subdev())
+		printf("\ttest invalid ioctls: %s\n", ok(testInvalidIoctls(&node, 'V')));
 
 	if (node.is_media()) {
 		printf("\ttest MEDIA_IOC_DEVICE_INFO: %s\n",
 		       ok(testMediaDeviceInfo(&node)));
+		printf("\ttest invalid ioctls: %s\n", ok(testInvalidIoctls(&node, '|')));
 	}
 	printf("\n");
 
@@ -1087,6 +1125,8 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 	case MEDIA_TYPE_SUBDEV:
 		printf("\ttest second %s open: %s\n", node.device,
 		       ok(node2.subdev_open(node.device, false) >= 0 ? 0 : errno));
+		if (node2.g_fd() >= 0)
+			printf("\ttest VIDIOC_SUBDEV_QUERYCAP: %s\n", ok(testSubDevCap(&node2)));
 		break;
 	case MEDIA_TYPE_MEDIA:
 		printf("\ttest second %s open: %s\n", node.device,
@@ -1109,8 +1149,8 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 
 	printf("\ttest for unlimited opens: %s\n",
 		ok(testUnlimitedOpens(&node)));
-	printf("\n");
 
+	printf("\n");
 	storeState(&node);
 
 	/* register signal handler for interrupt signal, to exit gracefully */
@@ -1133,13 +1173,10 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 			       node.topology->num_links);
 		printf("\ttest MEDIA_IOC_ENUM_ENTITIES/LINKS: %s\n", ok(testMediaEnum(&node)));
 		printf("\ttest MEDIA_IOC_SETUP_LINK: %s\n", ok(testMediaSetupLink(&node)));
-		printf("\ttest invalid ioctls: %s\n", ok(testInvalidIoctls(&node, '|')));
 		printf("\n");
 		goto show_total;
 	}
 
-	printf("\ttest invalid ioctls: %s\n", ok(testInvalidIoctls(&node, 'V')));
-
 	/* Debug ioctls */
 
 	printf("Debug ioctls:\n");
@@ -1249,11 +1286,12 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 		node.frmsizes.clear();
 		node.frmsizes_count.clear();
 		node.has_frmintervals = false;
+		node.has_enc_cap_frame_interval = false;
 		node.valid_buftypes = 0;
 		node.valid_memorytype = 0;
 		node.buf_caps = 0;
-		for (unsigned idx = 0; idx < V4L2_BUF_TYPE_LAST + 1; idx++)
-			node.buftype_pixfmts[idx].clear();
+		for (auto &buftype_pixfmt : node.buftype_pixfmts)
+			buftype_pixfmt.clear();
 
 		if (max_io) {
 			sprintf(suffix, " (%s %u)",
@@ -1285,7 +1323,11 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 		printf("\ttest VIDIOC_(UN)SUBSCRIBE_EVENT/DQEVENT: %s\n", ok(testEvents(&node)));
 		printf("\ttest VIDIOC_G/S_JPEGCOMP: %s\n", ok(testJpegComp(&node)));
 		printf("\tStandard Controls: %d Private Controls: %d\n",
-				node.std_controls, node.priv_controls);
+		       node.std_controls - node.std_compound_controls,
+		       node.priv_controls - node.priv_compound_controls);
+		if (node.std_compound_controls || node.priv_compound_controls)
+			printf("\tStandard Compound Controls: %d Private Compound Controls: %d\n",
+			       node.std_compound_controls, node.priv_compound_controls);
 		printf("\n");
 
 		/* Format ioctls */
@@ -1322,6 +1364,8 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 		if (node.can_capture)
 			node.set_interval(min_period);
 		printf("\ttest Requests: %s\n", ok(testRequests(&node, options[OptStreaming])));
+		if (sizeof(void *) == 4)
+			printf("\ttest TIME32/64: %s\n", ok(testTime32_64(&node)));
 		// Reopen after each streaming test to reset the streaming state
 		// in case of any errors in the preceeding test.
 		node.reopen();
@@ -1433,6 +1477,14 @@ void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_
 	 * 	 S_SELECTION flags tests
 	 */
 
+	if (is_vivid &&
+	    node.controls.find(VIVID_CID_DISCONNECT) != node.controls.end()) {
+		if (node.node2)
+			node.node2->close();
+		node.node2 = nullptr;
+		printf("\ttest Disconnect: %s\n\n", ok(testVividDisconnect(&node)));
+	}
+
 	restoreState();
 
 show_total:
@@ -1476,31 +1528,12 @@ int main(int argc, char **argv)
 	int ch;
 	std::string device("/dev/video0");
 	std::string expbuf_device;	/* --expbuf-device device */
-	struct utsname uts;
-	int v1, v2, v3;
 	unsigned frame_count = 60;
 	unsigned all_fmt_frame_count = 0;
 	char short_options[26 * 2 * 3 + 1];
 	char *value, *subs;
 	int idx = 0;
 
-#ifdef SHA
-#define STR(x) #x
-#define STRING(x) STR(x)
-	printf("v4l2-compliance SHA: %s", STRING(SHA));
-#else
-	printf("v4l2-compliance SHA: not available\n");
-#endif
-
-	printf(", %zd bits, %zd-bit time_t\n", sizeof(void *) * 8, sizeof(time_t) * 8);
-	uname(&uts);
-	sscanf(uts.release, "%d.%d.%d", &v1, &v2, &v3);
-	if (v1 == 2 && v2 == 6)
-		kernel_version = v3;
-	if (kernel_version)
-		printf("Running on 2.6.%d\n", kernel_version);
-	printf("\n");
-
 	if (!env_media_apps_color || !strcmp(env_media_apps_color, "auto"))
 		show_colors = isatty(STDOUT_FILENO);
 	else if (!strcmp(env_media_apps_color, "always"))
@@ -1580,7 +1613,7 @@ int main(int argc, char **argv)
 			break;
 		case OptStreaming:
 			if (optarg)
-				frame_count = strtoul(optarg, NULL, 0);
+				frame_count = strtoul(optarg, nullptr, 0);
 			break;
 		case OptStreamFrom:
 		case OptStreamFromHdr: {
@@ -1588,7 +1621,7 @@ int main(int argc, char **argv)
 			bool has_hdr = ch == OptStreamFromHdr;
 
 			if (equal == optarg)
-				equal = NULL;
+				equal = nullptr;
 			if (equal) {
 				*equal = '\0';
 				stream_from_map[optarg] = equal + 1;
@@ -1601,16 +1634,16 @@ int main(int argc, char **argv)
 		}
 		case OptStreamAllFormats:
 			if (optarg)
-				all_fmt_frame_count = strtoul(optarg, NULL, 0);
+				all_fmt_frame_count = strtoul(optarg, nullptr, 0);
 			break;
 		case OptStreamAllColorTest:
 			subs = optarg;
 			while (*subs != '\0') {
-				static const char *const subopts[] = {
+				static constexpr const char *subopts[] = {
 					"color",
 					"skip",
 					"perc",
-					NULL
+					nullptr
 				};
 
 				switch (parse_subopt(&subs, subopts, &value)) {
@@ -1627,10 +1660,10 @@ int main(int argc, char **argv)
 					}
 					break;
 				case 1:
-					color_skip = strtoul(value, 0L, 0);
+					color_skip = strtoul(value, nullptr, 0);
 					break;
 				case 2:
-					color_perc = strtoul(value, 0L, 0);
+					color_perc = strtoul(value, nullptr, 0);
 					if (color_perc == 0)
 						color_perc = 90;
 					if (color_perc > 100)
@@ -1669,6 +1702,9 @@ int main(int argc, char **argv)
 		case OptNoProgress:
 			no_progress = true;
 			break;
+		case OptVersion:
+			print_sha();
+			std::exit(EXIT_SUCCESS);
 		case ':':
 			fprintf(stderr, "Option `%s' requires a value\n",
 				argv[optind]);
@@ -1690,6 +1726,10 @@ int main(int argc, char **argv)
 		usage();
 		std::exit(EXIT_FAILURE);
 	}
+
+	print_sha();
+	printf("\n");
+
 	bool direct = !options[OptUseWrapper];
 	int fd;
 
