@@ -21,12 +21,10 @@
 #ifndef _V4L2_COMPLIANCE_H_
 #define _V4L2_COMPLIANCE_H_
 
-#include <stdarg.h>
-#include <cerrno>
-#include <string>
 #include <map>
 #include <set>
-#include <map>
+#include <string>
+
 #include <linux/videodev2.h>
 #include <linux/v4l2-subdev.h>
 #include <linux/media.h>
@@ -56,9 +54,11 @@ extern bool no_progress;
 extern bool exit_on_fail;
 extern bool exit_on_warn;
 extern bool is_vivid; // We're testing the vivid driver
+extern bool is_uvcvideo; // We're testing the uvc driver
 extern int kernel_version;
 extern int media_fd;
 extern unsigned warnings;
+extern bool has_mmu;
 
 enum poll_mode {
 	POLL_MODE_NONE,
@@ -73,14 +73,37 @@ enum poll_mode {
 #define STATELESS_ENCODER (1 << 4)
 #define STATELESS_DECODER (1 << 5)
 
+#define IS_ENCODER(node) ((node)->codec_mask & (JPEG_ENCODER | STATEFUL_ENCODER | STATELESS_ENCODER))
+#define IS_DECODER(node) ((node)->codec_mask & (JPEG_DECODER | STATEFUL_DECODER | STATELESS_DECODER))
+
+#define V4L2_CTRL_CLASS_VIVID		0x00f00000
+#define VIVID_CID_VIVID_BASE		(V4L2_CTRL_CLASS_VIVID | 0xf000)
+#define VIVID_CID_DISCONNECT		(VIVID_CID_VIVID_BASE + 65)
+#define VIVID_CID_DQBUF_ERROR		(VIVID_CID_VIVID_BASE + 66)
+#define VIVID_CID_QUEUE_SETUP_ERROR	(VIVID_CID_VIVID_BASE + 67)
+#define VIVID_CID_BUF_PREPARE_ERROR	(VIVID_CID_VIVID_BASE + 68)
+#define VIVID_CID_START_STR_ERROR	(VIVID_CID_VIVID_BASE + 69)
+#define VIVID_CID_QUEUE_ERROR		(VIVID_CID_VIVID_BASE + 70)
+#define VIVID_CID_REQ_VALIDATE_ERROR	(VIVID_CID_VIVID_BASE + 72)
+
+#define VIVID_CID_CUSTOM_BASE		(V4L2_CID_USER_BASE | 0xf000)
+#define VIVID_CID_INTEGER64		(VIVID_CID_CUSTOM_BASE + 3)
+#define VIVID_CID_STRING		(VIVID_CID_CUSTOM_BASE + 5)
+#define VIVID_CID_AREA			(VIVID_CID_CUSTOM_BASE + 11)
+#define VIVID_CID_RO_INTEGER		(VIVID_CID_CUSTOM_BASE + 12)
+#define VIVID_CID_U32_DYN_ARRAY		(VIVID_CID_CUSTOM_BASE + 13)
+#define VIVID_CID_U8_PIXEL_ARRAY	(VIVID_CID_CUSTOM_BASE + 14)
+
+#define PIXEL_ARRAY_DIV 16
+
 struct test_query_ext_ctrl: v4l2_query_ext_ctrl {
 	__u64 menu_mask;
 };
 
-typedef std::map<__u32, struct test_query_ext_ctrl> qctrl_map;
-typedef std::map<__u32, __u32> pixfmt_map;
-typedef std::set<__u64> frmsizes_set;
-typedef std::map<__u32, unsigned> frmsizes_count_map;
+using qctrl_map = std::map<__u32, test_query_ext_ctrl>;
+using pixfmt_map = std::map<__u32, __u32>;
+using frmsizes_set = std::set<__u64>;
+using frmsizes_count_map = std::map<__u32, unsigned>;
 
 struct base_node;
 
@@ -94,7 +117,9 @@ struct base_node {
 	bool is_meta;
 	bool is_touch;
 	bool is_m2m;
+	bool is_io_mc;
 	bool is_planar;
+	bool is_ro_subdev;
 	bool can_capture;
 	bool can_output;
 	bool can_scale;
@@ -102,6 +127,7 @@ struct base_node {
 	struct node *node2;	/* second open filehandle */
 	bool has_outputs;
 	bool has_inputs;
+	bool has_media;
 	unsigned tuners;
 	unsigned modulators;
 	unsigned inputs;
@@ -117,6 +143,7 @@ struct base_node {
 	__u32 media_version;
 	std::string media_bus_info;
 	media_entity_desc entity;
+	__u32 function;
 	media_pad_desc *pads;
 	media_link_desc *links;
 	media_v2_topology *topology;
@@ -136,10 +163,13 @@ struct base_node {
 	frmsizes_set frmsizes;
 	frmsizes_count_map frmsizes_count;
 	bool has_frmintervals;
+	bool has_enc_cap_frame_interval;
 	__u32 valid_buftypes;
 	__u32 valid_buftype;
 	__u32 valid_memorytype;
 	bool supports_orphaned_bufs;
+	// support for this was introduced in 5.9
+	bool might_support_cache_hints;
 };
 
 struct node : public base_node, public cv4l_fd {
@@ -205,17 +235,17 @@ private:
 			       COLOR_BOLD("warn") : "warn",	\
 			       __FILE__, __LINE__, ##args);	\
 		if (exit_on_warn)				\
-			exit(1);				\
+			std::exit(EXIT_FAILURE);		\
 	} while (0)
 
-#define warn_once(fmt, args...)						\
-	do {								\
-		static bool show;					\
-									\
-		if (!show) {						\
-			show = true;					\
-			warn(fmt, ##args);				\
-		}							\
+#define warn_once(fmt, args...)				\
+	do {						\
+		static bool show;			\
+							\
+		if (!show) {				\
+			show = true;			\
+			warn(fmt, ##args);		\
+		}					\
 	} while (0)
 
 #define warn_on_test(test) 				\
@@ -224,12 +254,26 @@ private:
 			warn("%s\n", #test);		\
 	} while (0)
 
+#define warn_once_on_test(test) 			\
+	do {						\
+		if (test)				\
+			warn_once("%s\n", #test);	\
+	} while (0)
+
+#define warn_or_info(is_info, fmt, args...) 		\
+	do {						\
+		if (is_info)				\
+			info(fmt, ##args);		\
+		else					\
+			warn(fmt, ##args);		\
+	} while (0)
+
 #define fail(fmt, args...) 						\
 ({ 									\
 	printf("\t\t%s: %s(%d): " fmt, show_colors ?			\
 	       COLOR_RED("fail") : "fail", __FILE__, __LINE__, ##args);	\
 	if (exit_on_fail)						\
-		exit(1);						\
+		std::exit(EXIT_FAILURE);				\
 	1;								\
 })
 
@@ -239,6 +283,12 @@ private:
 			return fail("%s\n", #test);	\
 	} while (0)
 
+#define fail_on_test_val(test, v)				\
+	do {							\
+	 	if (test)					\
+			return fail("%s (got %d)\n", #test, v);	\
+	} while (0)
+
 static inline int check_fract(const struct v4l2_fract *f)
 {
 	if (f->numerator && f->denominator)
@@ -260,7 +310,7 @@ int check_0(const void *p, int len);
 int restoreFormat(struct node *node);
 void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_node, media_type type,
 	      unsigned frame_count, unsigned all_fmt_frame_count);
-std::string stream_from(std::string pixelformat, bool &use_hdr);
+std::string stream_from(const std::string &pixelformat, bool &use_hdr);
 
 // Media Controller ioctl tests
 int testMediaDeviceInfo(struct node *node);
@@ -295,6 +345,7 @@ int testQueryControls(struct node *node);
 int testSimpleControls(struct node *node);
 int testExtendedControls(struct node *node);
 int testEvents(struct node *node);
+int testVividDisconnect(struct node *node);
 int testJpegComp(struct node *node);
 
 // I/O configuration ioctl tests
@@ -321,6 +372,7 @@ int testEncIndex(struct node *node);
 int testDecoder(struct node *node);
 
 // SubDev ioctl tests
+int testSubDevCap(struct node *node);
 int testSubDevEnum(struct node *node, unsigned which, unsigned pad);
 int testSubDevFormat(struct node *node, unsigned which, unsigned pad);
 int testSubDevSelection(struct node *node, unsigned which, unsigned pad);
@@ -331,6 +383,10 @@ int testReqBufs(struct node *node);
 int testReadWrite(struct node *node);
 int testExpBuf(struct node *node);
 int testBlockingWait(struct node *node);
+
+// 32-bit architecture, 32/64-bit time_t tests
+int testTime32_64(struct node *node);
+
 /*
  * struct node node:
  * 	the current media node being tested
