@@ -3,26 +3,25 @@
  * Copyright 2018 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  */
 
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <inttypes.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <errno.h>
-#include <sys/ioctl.h>
-#include <sys/time.h>
-#include <sys/sysmacros.h>
+#include <cstdint>
+#include <cstring>
+#include <fstream>
+#include <iostream>
+
 #include <dirent.h>
+#include <fcntl.h>
+#include <sys/ioctl.h>
+#include <sys/stat.h>
+#include <sys/sysmacros.h>
+#include <unistd.h>
 
 #include <linux/media.h>
 
-#include <iostream>
-#include <fstream>
 #include <media-info.h>
+#ifndef __linux__
+#include <linux/videodev2.h>
+#include <fcntl.h>
+#endif
 
 static std::string num2s(unsigned num, bool is_hex = true)
 {
@@ -35,7 +34,7 @@ static std::string num2s(unsigned num, bool is_hex = true)
 	return buf;
 }
 
-static struct {
+static constexpr struct {
 	const char *devname;
 	enum media_type type;
 } media_types[] = {
@@ -51,7 +50,7 @@ static struct {
 	{ "dvr", MEDIA_TYPE_DVB_DVR },
 	{ "net", MEDIA_TYPE_DVB_NET },
 	{ "ca", MEDIA_TYPE_DTV_CA },
-	{ NULL, MEDIA_TYPE_UNKNOWN }
+	{ nullptr, MEDIA_TYPE_UNKNOWN }
 };
 
 media_type mi_media_detect_type(const char *device)
@@ -60,7 +59,7 @@ media_type mi_media_detect_type(const char *device)
 
 	if (stat(device, &sb) == -1)
 		return MEDIA_TYPE_CANT_STAT;
-
+#ifdef __linux__
 	std::string uevent_path("/sys/dev/char/");
 
 	uevent_path += num2s(major(sb.st_rdev), false) + ":" +
@@ -96,6 +95,23 @@ media_type mi_media_detect_type(const char *device)
 	}
 
 	uevent_file.close();
+#else // Not Linux
+	int fd = open(device, O_RDONLY);
+	if (fd >= 0) {
+		struct v4l2_capability caps;
+		int error = ioctl(fd, VIDIOC_QUERYCAP, &caps);
+		close(fd);
+		if (error == 0) {
+			if (caps.device_caps & V4L2_CAP_VIDEO_CAPTURE) {
+				return MEDIA_TYPE_VIDEO;
+			} else if (caps.device_caps & V4L2_CAP_VBI_CAPTURE) {
+				return MEDIA_TYPE_VBI;
+			} else if (caps.device_caps & V4L2_CAP_RADIO) {
+				return MEDIA_TYPE_RADIO;
+			}
+		}
+	}
+#endif
 	return MEDIA_TYPE_UNKNOWN;
 }
 
@@ -124,10 +140,10 @@ std::string mi_media_get_device(__u32 major, __u32 minor)
 	return "";
 }
 
-typedef struct {
+struct flag_def {
 	unsigned flag;
 	const char *str;
-} flag_def;
+};
 
 static std::string flags2s(unsigned val, const flag_def *def)
 {
@@ -172,12 +188,12 @@ std::string mi_get_devpath_from_dev_t(dev_t dev)
 
 	FILE *uevent_fd = fopen(media_uevent_path.c_str(), "r");
 
-	if (uevent_fd == NULL) {
+	if (uevent_fd == nullptr) {
 		fprintf(stderr, "failed to open %s\n", media_uevent_path.c_str());
 		return "";
 	}
 
-	char *line = NULL;
+	char *line = nullptr;
 	size_t size = 0;
 	std::string devpath;
 
@@ -216,7 +232,7 @@ int mi_get_media_fd(int fd, const char *bus_info)
 	DIR *dp;
 	struct dirent *ep;
 	dp = opendir(media_path.c_str());
-	if (dp == NULL)
+	if (dp == nullptr)
 		return -1;
 	media_path[0] = 0;
 	while ((ep = readdir(dp))) {
@@ -240,10 +256,10 @@ int mi_get_media_fd(int fd, const char *bus_info)
 	return media_fd;
 }
 
-static const flag_def entity_flags_def[] = {
+static constexpr flag_def entity_flags_def[] = {
 	{ MEDIA_ENT_FL_DEFAULT, "default" },
 	{ MEDIA_ENT_FL_CONNECTOR, "connector" },
-	{ 0, NULL }
+	{ 0, nullptr }
 };
 
 std::string mi_entflags2s(__u32 flags)
@@ -251,7 +267,7 @@ std::string mi_entflags2s(__u32 flags)
 	return flags2s(flags, entity_flags_def);
 }
 
-static const flag_def interface_types_def[] = {
+static constexpr flag_def interface_types_def[] = {
 	{ MEDIA_INTF_T_DVB_FE, "DVB Front End" },
 	{ MEDIA_INTF_T_DVB_DEMUX, "DVB Demuxer" },
 	{ MEDIA_INTF_T_DVB_DVR, "DVB DVR" },
@@ -273,7 +289,7 @@ static const flag_def interface_types_def[] = {
 	{ MEDIA_INTF_T_ALSA_HWDEP, "ALSA HWDEP" },
 	{ MEDIA_INTF_T_ALSA_SEQUENCER, "ALSA Sequencer" },
 	{ MEDIA_INTF_T_ALSA_TIMER, "ALSA Timer" },
-	{ 0, NULL }
+	{ 0, nullptr }
 };
 
 std::string mi_ifacetype2s(__u32 type)
@@ -284,7 +300,7 @@ std::string mi_ifacetype2s(__u32 type)
 	return "FAIL: Unknown (" + num2s(type) + ")";
 }
 
-static const flag_def entity_functions_def[] = {
+static constexpr flag_def entity_functions_def[] = {
 	{ MEDIA_ENT_F_UNKNOWN, "FAIL: Uninitialized Function" },
 	{ MEDIA_ENT_F_V4L2_SUBDEV_UNKNOWN, "FAIL: Unknown V4L2 Sub-Device" },
 	{ MEDIA_ENT_T_DEVNODE_UNKNOWN, "FAIL: Unknown Device Node" },
@@ -320,7 +336,7 @@ static const flag_def entity_functions_def[] = {
 	{ MEDIA_ENT_F_PROC_VIDEO_ENCODER, "Video Encoder" },
 	{ MEDIA_ENT_F_VID_MUX, "Video Muxer" },
 	{ MEDIA_ENT_F_VID_IF_BRIDGE, "Video Interface Bridge" },
-	{ 0, NULL }
+	{ 0, nullptr }
 };
 
 std::string mi_entfunction2s(__u32 function, bool *is_invalid)
@@ -368,7 +384,6 @@ bool mi_func_requires_intf(__u32 function)
 	case MEDIA_ENT_F_DTV_DEMOD:
 	case MEDIA_ENT_F_TS_DEMUX:
 	case MEDIA_ENT_F_DTV_CA:
-	case MEDIA_ENT_F_IO_DTV:
 	case MEDIA_ENT_F_IO_V4L:
 	case MEDIA_ENT_F_IO_VBI:
 	case MEDIA_ENT_F_IO_SWRADIO:
@@ -378,11 +393,11 @@ bool mi_func_requires_intf(__u32 function)
 	}
 }
 
-static const flag_def pad_flags_def[] = {
+static constexpr flag_def pad_flags_def[] = {
 	{ MEDIA_PAD_FL_SINK, "Sink" },
 	{ MEDIA_PAD_FL_SOURCE, "Source" },
 	{ MEDIA_PAD_FL_MUST_CONNECT, "Must Connect" },
-	{ 0, NULL }
+	{ 0, nullptr }
 };
 
 std::string mi_padflags2s(__u32 flags)
@@ -390,11 +405,11 @@ std::string mi_padflags2s(__u32 flags)
 	return flags2s(flags, pad_flags_def);
 }
 
-static const flag_def link_flags_def[] = {
+static constexpr flag_def link_flags_def[] = {
 	{ MEDIA_LNK_FL_ENABLED, "Enabled" },
 	{ MEDIA_LNK_FL_IMMUTABLE, "Immutable" },
 	{ MEDIA_LNK_FL_DYNAMIC, "Dynamic" },
-	{ 0, NULL }
+	{ 0, nullptr }
 };
 
 std::string mi_linkflags2s(__u32 flags)
@@ -408,13 +423,16 @@ std::string mi_linkflags2s(__u32 flags)
 		return "Data" + s;
 	case MEDIA_LNK_FL_INTERFACE_LINK:
 		return "Interface" + s;
+	case MEDIA_LNK_FL_ANCILLARY_LINK:
+		return "Ancillary" + s;
 	default:
 		return "Unknown (" + num2s(flags) + ")" + s;
 	}
 }
 
 static __u32 read_topology(int media_fd, __u32 major, __u32 minor,
-			   __u32 media_version, bool *is_invalid)
+			   __u32 media_version, bool *is_invalid,
+			   __u32 *function)
 {
 	media_v2_topology topology;
 	unsigned i, j;
@@ -428,10 +446,10 @@ static __u32 read_topology(int media_fd, __u32 major, __u32 minor,
 	media_v2_pad v2_pads[topology.num_pads];
 	media_v2_link v2_links[topology.num_links];
 
-	topology.ptr_entities = (__u64)v2_ents;
-	topology.ptr_interfaces = (__u64)v2_ifaces;
-	topology.ptr_pads = (__u64)v2_pads;
-	topology.ptr_links = (__u64)v2_links;
+	topology.ptr_entities = (uintptr_t)v2_ents;
+	topology.ptr_interfaces = (uintptr_t)v2_ifaces;
+	topology.ptr_pads = (uintptr_t)v2_pads;
+	topology.ptr_links = (uintptr_t)v2_links;
 	if (ioctl(media_fd, MEDIA_IOC_G_TOPOLOGY, &topology))
 		return 0;
 	for (i = 0; i < topology.num_interfaces; i++)
@@ -443,9 +461,11 @@ static __u32 read_topology(int media_fd, __u32 major, __u32 minor,
 			major, minor);
 		if (is_invalid)
 			*is_invalid = true;
+		if (function)
+			*function = MEDIA_ENT_F_UNKNOWN;
 		return MEDIA_ENT_F_UNKNOWN;
 	}
-	media_v2_interface &iface = v2_ifaces[i];
+	const media_v2_interface &iface = v2_ifaces[i];
 	for (i = 0; i < topology.num_links; i++) {
 		__u32 type = v2_links[i].flags & MEDIA_LNK_FL_LINK_TYPE;
 
@@ -492,13 +512,13 @@ static __u32 read_topology(int media_fd, __u32 major, __u32 minor,
 	if (is_radio)
 		return MEDIA_ENT_F_UNKNOWN;
 
-	media_v2_entity &ent = v2_ents[i];
+	const media_v2_entity &ent = v2_ents[i];
 	printf("Entity Info:\n");
 	printf("\tID               : 0x%08x (%u)\n", ent.id, ent.id);
 	printf("\tName             : %s\n", ent.name);
 	printf("\tFunction         : %s\n", mi_entfunction2s(ent.function, is_invalid).c_str());
 	if (MEDIA_V2_ENTITY_HAS_FLAGS(media_version) && ent.flags)
-		printf("\tFlags         : %s\n", mi_entflags2s(ent.flags).c_str());
+		printf("\tFlags            : %s\n", mi_entflags2s(ent.flags).c_str());
 
 	// Yes, I know, lots of nested for-loops. If we get really complex
 	// devices with such large topologies that this becomes too inefficient
@@ -518,7 +538,7 @@ static __u32 read_topology(int media_fd, __u32 major, __u32 minor,
 			__u32 type = link.flags & MEDIA_LNK_FL_LINK_TYPE;
 			__u32 remote_pad;
 			__u32 remote_ent_id = 0;
-			const media_v2_entity *remote_ent = NULL;
+			const media_v2_entity *remote_ent = nullptr;
 
 			if (type != MEDIA_LNK_FL_DATA_LINK ||
 			    (link.source_id != pad.id && link.sink_id != pad.id))
@@ -549,15 +569,18 @@ static __u32 read_topology(int media_fd, __u32 major, __u32 minor,
 					remote_ent_id);
 				return ent.id;
 			}
-			printf("\t  Link 0x%08x: %s remote pad 0x%x of entity '%s': %s\n",
+			printf("\t  Link 0x%08x: %s remote pad 0x%x of entity '%s' (%s): %s\n",
 			       link.id, is_sink ? "from" : "to", remote_pad,
-			       remote_ent->name, mi_linkflags2s(link.flags).c_str());
+			       remote_ent->name, mi_entfunction2s(remote_ent->function).c_str(),
+			       mi_linkflags2s(link.flags).c_str());
+			if (function && !*function)
+				*function = remote_ent->function;
 		}
 	}
 	return ent.id;
 }
 
-__u32 mi_media_info_for_fd(int media_fd, int fd, bool *is_invalid)
+__u32 mi_media_info_for_fd(int media_fd, int fd, bool *is_invalid, __u32 *function)
 {
 	struct media_device_info mdinfo;
 	struct stat sb;
@@ -566,6 +589,9 @@ __u32 mi_media_info_for_fd(int media_fd, int fd, bool *is_invalid)
 	if (is_invalid)
 		*is_invalid = false;
 
+	if (function)
+		*function = MEDIA_ENT_F_UNKNOWN;
+
 	if (ioctl(media_fd, MEDIA_IOC_DEVICE_INFO, &mdinfo))
 		return 0;
 
@@ -593,11 +619,11 @@ __u32 mi_media_info_for_fd(int media_fd, int fd, bool *is_invalid)
 
 	if (fstat(fd, &sb) == -1) {
 		fprintf(stderr, "failed to stat file\n");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 
 	ent_id = read_topology(media_fd, major(sb.st_rdev), minor(sb.st_rdev),
-			       mdinfo.media_version, is_invalid);
+			       mdinfo.media_version, is_invalid, function);
 	if (ent_id)
 		return ent_id;
 
