@@ -390,6 +390,9 @@ static bool fill_subset(const struct v4l2_query_ext_ctrl &qc, ctrl_subset &subse
 		subset.size[d] = qc.dims[d];
 	}
 
+	if (qc.flags & V4L2_CTRL_FLAG_DYNAMIC_ARRAY)
+		subset.size[0] = qc.elems;
+
 	std::string s = name2var(qc.name);
 
 	if (ctrl_subsets.find(s) != ctrl_subsets.end()) {
@@ -489,6 +492,8 @@ static void print_value(int fd, const v4l2_query_ext_ctrl &qc, const v4l2_ext_co
 			memset(&subset, 0, sizeof(subset));
 			for (unsigned i = 0; i < qc.nr_of_dims; i++)
 				subset.size[i] = qc.dims[i];
+			if (qc.flags & V4L2_CTRL_FLAG_DYNAMIC_ARRAY)
+				subset.size[0] = qc.elems;
 		}
 		print_array(qc, ctrl, subset);
 		return;
@@ -623,9 +628,27 @@ static void print_qctrl(int fd, const v4l2_query_ext_ctrl &qc,
 	case V4L2_CTRL_TYPE_H264_PRED_WEIGHTS:
 		printf("%31s %#8.8x (h264-pred-weights):", s.c_str(), qc.id);
 		break;
+	case V4L2_CTRL_TYPE_HEVC_SPS:
+		printf("%31s %#8.8x (hevc-sps):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_HEVC_PPS:
+		printf("%31s %#8.8x (hevc-sps):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_HEVC_SLICE_PARAMS:
+		printf("%31s %#8.8x (hevc-slice-params):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_HEVC_SCALING_MATRIX:
+		printf("%31s %#8.8x (hevc-scaling-matrix):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_HEVC_DECODE_PARAMS:
+		printf("%31s %#8.8x (hevc-decode-params):", s.c_str(), qc.id);
+		break;
 	case V4L2_CTRL_TYPE_VP8_FRAME:
 		printf("%31s %#8.8x (vp8-frame):", s.c_str(), qc.id);
 		break;
+	case V4L2_CTRL_TYPE_VP9_FRAME:
+		printf("%31s %#8.8x (vp9-frame):", s.c_str(), qc.id);
+		break;
 	case V4L2_CTRL_TYPE_MPEG2_QUANTISATION:
 		printf("%31s %#8.8x (mpeg2-quantisation):", s.c_str(), qc.id);
 		break;
@@ -647,6 +670,8 @@ static void print_qctrl(int fd, const v4l2_query_ext_ctrl &qc,
 		printf(" value=");
 		print_value(fd, qc, ctrl, false, false);
 	} else {
+		if (qc.flags & V4L2_CTRL_FLAG_DYNAMIC_ARRAY)
+			printf(" elems=%u", qc.elems);
 		printf(" dims=");
 		for (i = 0; i < qc.nr_of_dims; i++)
 			printf("[%u]", qc.dims[i]);
@@ -698,6 +723,7 @@ static int print_control(int fd, struct v4l2_query_ext_ctrl &qctrl, bool show_me
 	ctrls.controls = &ext_ctrl;
 	if (qctrl.type == V4L2_CTRL_TYPE_INTEGER64 ||
 	    qctrl.type == V4L2_CTRL_TYPE_STRING ||
+	    qctrl.nr_of_dims ||
 	    qctrl.type >= V4L2_CTRL_COMPOUND_TYPES ||
 	    (V4L2_CTRL_ID2WHICH(qctrl.id) != V4L2_CTRL_CLASS_USER &&
 	     qctrl.id < V4L2_CID_PRIVATE_BASE)) {
@@ -877,7 +903,7 @@ void common_process_controls(cv4l_fd &fd)
 	}
 }
 
-void common_control_event(const struct v4l2_event *ev)
+void common_control_event(int fd, const struct v4l2_event *ev)
 {
 	const struct v4l2_event_ctrl *ctrl;
 
@@ -901,6 +927,18 @@ void common_control_event(const struct v4l2_event *ev)
 			printf("\trange: min=%d max=%d step=%d default=%d\n",
 				ctrl->minimum, ctrl->maximum, ctrl->step, ctrl->default_value);
 	}
+	if (ctrl->changes & V4L2_EVENT_CTRL_CH_DIMENSIONS) {
+		v4l2_query_ext_ctrl qctrl = {};
+	
+		qctrl.id = ev->id;
+		if (!query_ext_ctrl_ioctl(fd, qctrl)) {
+			ctrl_str2q[name2var(qctrl.name)] = qctrl;
+			printf("\tdimensions: ");
+			for (unsigned i = 0; i < qctrl.nr_of_dims; i++)
+				printf("[%u]", qctrl.dims[i]);
+			printf("\n");
+		}
+	}
 }
 
 static bool parse_subset(char *optarg)
@@ -959,7 +997,7 @@ static bool parse_next_subopt(char **subs, char **value)
 	static char *const subopts[] = {
 	    nullptr
 	};
-	int opt = getsubopt(subs, subopts, value);
+	int opt = v4l_getsubopt(subs, subopts, value);
 
 	if (opt < 0 || *value)
 		return false;
@@ -1228,7 +1266,7 @@ void common_get(cv4l_fd &_fd)
 
 					if (qc.nr_of_dims) {
 						print_value(fd, qc, ctrl, true, true);
-						return;
+						continue;
 					}
 
 					printf("%s: ", name.c_str());
